<?php

namespace App\Http\Controllers\Api\V1;

use App\Enums\BillStatus;
use App\Http\Controllers\Controller;
use App\Http\Requests\BillRequest;
use App\Http\Resources\BillResource;
use App\Models\Bill;
use Illuminate\Http\JsonResponse;

class BillController extends Controller
{
    protected array $relations = [
        "saleOrder", "purchaseOrder", "customer", "items", "payment"
    ];

    /**
     * Display a listing of the bills.
     */
    public function index(): JsonResponse {
        $bills = $this->paginateOrGet(
            Bill::with($this->getRelations())->filter()->latest());

        return $this->respondWithResourceCollection(
            BillResource::collection($bills)
        );
    }

    /**
     * Store a newly created bill in storage.
     */
    public function store(BillRequest $request): JsonResponse {
        $bill = $this->transactional(function () use ($request) {
            $bill = Bill::create($request->validated());

            $this->attachOrSyncItems($bill, $request);

            $this->managePayment($bill, $request);

            return $bill;
        });


        return $this->respondCreated(
            new BillResource($bill),
            "Bill created successfully!"
        );
    }

    /**
     * A private method to handle saving the Bill and linking items.
     */
    private function attachOrSyncItems(Bill $bill, BillRequest $request, string $method = "attach"): void {
        // attach or sync the items with the pivot table, their quantities and amounts
        if ($request->has('items')) {
            $items = collect($request->validated('items'))->mapWithKeys(function ($item) {
                return [$item['id'] => ['quantity' => $item['quantity'], 'amount' => $item['amount']]];
            });

            // If it's an existing bill, sync the items; otherwise, attach
            $bill->items()->$method($items);
        }
    }

    private function managePayment(Bill $bill, BillRequest $request): void {
        if ($request->validated("status") === BillStatus::PAID->value) {
            $bill->payment()->updateOrCreate(
                ['id' => $bill->payment?->id], // condition for update
                // data to update/create
                [
                    'is_credit' => true,
                    'amount' => $bill->total_amount,
                    'mode' => $request->validated('mode')
                ]
            );
        }
    }

    /**
     * Display the specified bill.
     */
    public function show(Bill $bill): JsonResponse {
        $bill->loadMissing($this->getRelations());

        return $this->respondWithResource(new BillResource($bill));
    }

    /**
     * Update the specified bill in storage.
     */
    public function update(BillRequest $request, Bill $bill): JsonResponse {
        $this->transactional(function () use ($request, $bill) {
            $bill->update($request->validated());

            $this->attachOrSyncItems($bill, $request, "sync");

            $this->managePayment($bill, $request);
        });

        return $this->respondUpdated(
            new BillResource($bill),
            "Bill updated successfully!"
        );
    }

    /**
     * Remove the specified bill from storage.
     */
    public function destroy(Bill $bill): JsonResponse {
        $bill->delete();

        return $this->respondSuccess("Bill deleted successfully!");
    }
}
