<?php

namespace App\Http\Controllers\Api\V1;

use App\Http\Controllers\Controller;
use App\Http\Requests\JobCardRequest;
use App\Http\Resources\JobCardResource;
use App\Models\JobCard;
use Illuminate\Http\JsonResponse;

class JobCardController extends Controller
{
    protected array $relations = [
        "purchaseOrder", "items", "jobCardUpdates", "outsourcedJobs"
    ];

    /**
     * Display a listing of the jobCards.
     */
    public function index(): JsonResponse {
        $jobCards = $this->paginateOrGet(
            JobCard::with($this->getRelations())->filter()->latest());

        return $this->respondWithResourceCollection(
            JobCardResource::collection($jobCards)
        );
    }

    /**
     * Store a newly created jobCard in storage.
     */
    public function store(JobCardRequest $request): JsonResponse {
        $jobCard = JobCard::create($request->validated());

        $this->attachOrSyncItems($jobCard, $request);

        return $this->respondCreated(
            new JobCardResource($jobCard),
            "JobCard created successfully!"
        );
    }

    /**
     * A private method to handle saving the PurchaseOrder and linking items.
     */
    private function attachOrSyncItems(
        JobCard $jobCard,
        JobCardRequest $request,
        string $method = "attach"
    ): void {
        // attach or sync the items with the pivot table and their quantities
        if ($request->has('items')) {
            $items = collect($request->validated('items'))->mapWithKeys(function ($item) {
                return [$item['id'] => ['quantity' => $item['quantity']]];
            });

            // If it's an existing purchase order, sync the items; otherwise, attach
            $jobCard->items()->$method($items);
        }
    }

    /**
     * Display the specified jobCard.
     */
    public function show(JobCard $jobCard): JsonResponse {
        $jobCard->loadMissing($this->getRelations());

        return $this->respondWithResource(new JobCardResource($jobCard));
    }

    /**
     * Update the specified jobCard in storage.
     */
    public function update(JobCardRequest $request, JobCard $jobCard): JsonResponse {
        $jobCard->update($request->validated());

        $this->attachOrSyncItems($jobCard, $request, "sync");

        return $this->respondUpdated(
            new JobCardResource($jobCard),
            "JobCard updated successfully!"
        );
    }

    /**
     * Remove the specified jobCard from storage.
     */
    public function destroy(JobCard $jobCard): JsonResponse {
        $jobCard->delete();

        return $this->respondSuccess("JobCard deleted successfully!");
    }
}
