<?php

namespace App\Http\Controllers\Api\V1;

use App\Http\Controllers\Controller;
use App\Http\Requests\PurchaseOrderRequest;
use App\Http\Resources\PurchaseOrderResource;
use App\Models\PurchaseOrder;
use Illuminate\Http\JsonResponse;

class PurchaseOrderController extends Controller
{
    protected array $relations = [
        'factoryRelation', 'customer', 'items', 'jobCard'
    ];

    /**
     * Display a listing of the purchaseOrders.
     */
    public function index(): JsonResponse {
        $purchaseOrders = $this->paginateOrGet(
            PurchaseOrder::with($this->getRelations())->filter()->latest());

        return $this->respondWithResourceCollection(
            PurchaseOrderResource::collection($purchaseOrders)
        );
    }

    /**
     * Store a newly created purchaseOrder in storage.
     */
    public function store(PurchaseOrderRequest $request): JsonResponse {
        $purchaseOrder = PurchaseOrder::create($request->validated());

        $this->attachOrSyncItems($purchaseOrder, $request);

        return $this->respondCreated(
            new PurchaseOrderResource($purchaseOrder),
            "PurchaseOrder created successfully!"
        );
    }

    /**
     * A private method to handle saving the PurchaseOrder and linking items.
     */
    private function attachOrSyncItems(
        PurchaseOrder $purchaseOrder,
        PurchaseOrderRequest $request,
        string $method = "attach"
    ): void {
        // attach or sync the items with the pivot table, their quantities and amounts
        if ($request->has('items')) {
            $items = collect($request->validated('items'))->mapWithKeys(function ($item) {
                return [$item['id'] => ['quantity' => $item['quantity'], 'amount' => $item['amount']]];
            });

            // If it's an existing purchase order, sync the items; otherwise, attach
            $purchaseOrder->items()->$method($items);
        }
    }

    public function show(PurchaseOrder $purchaseOrder) {
        $purchaseOrder->loadMissing($this->getRelations());

        return $this->respondWithResource(
            new PurchaseOrderResource($purchaseOrder)
        );
    }

    /**
     * Update the specified purchaseOrder in storage.
     */
    public function update(PurchaseOrderRequest $request, PurchaseOrder $purchaseOrder): JsonResponse {
        $purchaseOrder->update($request->validated());

        $this->attachOrSyncItems($purchaseOrder, $request, "sync");

        // Step 3: Return the response
        return $this->respondUpdated(
            new PurchaseOrderResource($purchaseOrder),
            "PurchaseOrder updated successfully!"
        );
    }

    /**
     * Remove the specified purchaseOrder from storage.
     */
    public function destroy(PurchaseOrder $purchaseOrder): JsonResponse {
        $purchaseOrder->delete();

        return $this->respondSuccess("PurchaseOrder deleted successfully!");
    }
}


