<?php

namespace App\Http\Controllers\Api\V1;

use App\Http\Controllers\Controller;
use App\Http\Requests\SupplyRequest;
use App\Http\Resources\SupplyResource;
use App\Models\Supply;
use Illuminate\Http\JsonResponse;

class SupplyController extends Controller
{
    protected array $relations = [
        "vendor", "item"
    ];

    /**
     * Display a listing of the supplies.
     */
    public function index(): JsonResponse {
        $supplies = $this->paginateOrGet(
            Supply::with($this->getRelations())->latest());

        return $this->respondWithResourceCollection(
            SupplyResource::collection($supplies)
        );
    }

    /**
     * Store a newly created supply in storage.
     */
    public function store(SupplyRequest $request): JsonResponse {
        $supply = Supply::create($request->validated());
        $supply->item()->increment('stock_level', $supply->quantity);

        return $this->respondCreated(
            new SupplyResource($supply),
            "Supply created successfully!"
        );
    }

    /**
     * Update the specified supply in storage.
     */
    public function update(SupplyRequest $request, Supply $supply): JsonResponse {
        $difference = $request->validated("quantity") - $supply->quantity;
        $supply->update($request->validated());
        $supply->item()->increment('stock_level', $difference);

        return $this->respondUpdated(
            new SupplyResource($supply),
            "Supply updated successfully!"
        );
    }

    /**
     * Display the specified supply.
     */
    public function show(Supply $supply): JsonResponse {
        $supply->loadMissing($this->relations);

        return $this->respondWithResource(new SupplyResource($supply));
    }

    /**
     * Remove the specified supply from storage.
     */
    public function destroy(Supply $supply): JsonResponse {
        $supply->item()->decrement('stock_level', $supply->quantity);
        $supply->delete();

        return $this->respondSuccess("Supply deleted successfully!");
    }
}
